# -*- coding: utf-8 -*-
# Generated by Django 1.9.1 on 2016-09-27 16:12
from __future__ import unicode_literals

from django.db import migrations, models


class Migration(migrations.Migration):

    dependencies = [
        ('dashboards', '0002_auto_20160922_1634'),
    ]

    operations = [
        migrations.AlterField(
            model_name='widget',
            name='data',
            field=models.ManyToManyField(blank=True, help_text=b"Data sources to pull from.<br/>'Last' metrics are advised for 'last' widgets. 'Sum' metrics are advised for all other widget types.<br/>", to='dashboards.WidgetData'),
        ),
        migrations.AlterField(
            model_name='widget',
            name='nulls',
            field=models.CharField(choices=[(b'omit', b'Omit'), (b'zeroize', b'Zeroize')], default=b'zeroize', help_text=b"What to do with intervals that have no data.<br/>'Omit' is advised for pie charts, 'Zeroize' is advised for all other widget types", max_length=20),
        ),
    ]
