# -*- coding: utf-8 -*-
# Generated by Django 1.9.1 on 2016-08-18 07:50
from __future__ import unicode_literals

from django.db import migrations, models


class Migration(migrations.Migration):

    dependencies = [
        ('services', '0003_auto_20160813_1717'),
    ]

    operations = [
        migrations.AlterField(
            model_name='userservicetoken',
            name='token',
            field=models.CharField(max_length=40),
        ),
    ]
