# -*- coding: utf-8 -*-
# Generated by Django 1.9.12 on 2017-03-08 08:31
from __future__ import unicode_literals

from django.db import migrations


class Migration(migrations.Migration):

    dependencies = [
        ('dashboards', '0004_definition'),
    ]

    operations = [
        migrations.AlterModelOptions(
            name='definition',
            options={},
        ),
    ]
